import type { Locale } from "@repo/i18n/config"

export const sum = (a: number, b: number): number => {
  return a + b
}

export const WIKIPEDIA_LOCALES = ["en", "fr"] as const
export type WikipediaLocale = (typeof WIKIPEDIA_LOCALES)[number]

export const fromLocaleToWikipediaLocale = (
  locale: Locale,
): WikipediaLocale => {
  return locale === "en-US" ? "en" : "fr"
}

export const getWikipediaLink = (locale: WikipediaLocale): string => {
  return `https://${locale}.wikipedia.org`
}

interface WikipediaQueryLinksResponse {
  continue?: {
    plcontinue: string
    continue: string
  }
  query: {
    pages: {
      [key: string]: {
        pageid: number
        ns: number
        title: string
        links: [
          {
            ns: number
            title: string
          },
        ]
      }
    }
  }
  limits: {
    links: number
  }
}

interface GetWikipediaPageInternalLinksInput {
  title: string
  locale: WikipediaLocale
}

interface GetWikipediaPageInternalLinksOutput {
  title: string
  links: string[]
}

export const getWikipediaPageInternalLinks = async (
  input: GetWikipediaPageInternalLinksInput,
): Promise<GetWikipediaPageInternalLinksOutput> => {
  const links: string[] = []
  let title = input.title
  let plcontinue: string | null = null

  const fetchLinks = async (): Promise<WikipediaQueryLinksResponse> => {
    const url = new URL("/w/api.php", getWikipediaLink(input.locale))
    url.searchParams.append("action", "query")
    url.searchParams.append("titles", title)
    url.searchParams.append("prop", "links")
    url.searchParams.append("pllimit", "max")
    url.searchParams.append("format", "json")
    url.searchParams.append("origin", "*")
    if (plcontinue != null) {
      url.searchParams.set("plcontinue", plcontinue)
    }
    const response = await fetch(url, {
      method: "GET",
    })
    if (!response.ok) {
      throw new Error(response.statusText)
    }
    const json = (await response.json()) as WikipediaQueryLinksResponse
    return json
  }

  do {
    try {
      const response = await fetchLinks()
      plcontinue = response?.continue?.plcontinue ?? null
      const pages = Object.keys(response.query.pages)
      const page = pages[0] ?? ""
      if (page === "-1" || page === "") {
        break
      }
      const pageData = response.query.pages[page]
      if (pageData == null) {
        break
      }
      title = pageData.title
      links.push(
        ...pageData.links.map((link) => {
          return link.title
        }),
      )
    } catch {
      break
    }
  } while (plcontinue != null)

  return {
    title,
    links,
  }
}
